unit DriveEcran_Imprimante;



{ Mise  jour du 23 03 2006

Cette unit procure les modules pour dessiner  l'cran sur une page contenue
dans un objet PaintBox et pour imprimer la mme chose  l'imprimante

 Il simule un aperu d'impression  l'cran. Cet aperu est le plus approchant et
 reprsente la taille de papier slectionn dans l'imprimante.

 Les nouveaut de cette mise  jour sont les suivantes:

 - Les paramtres de l'imprimante sont directement lues dans le driver de cette
 dernire. Les marges de non impression, la taille papier, la surface imprimable,
 etc sont donc directement accessibles dans une nouvelle structure.

 - Les coefficients cran et imprimante sont directement calculs en fonction des
 infos imprimante et cran.

 - Les orientations portrait et paysage, la taile du papier sont prises en compte
 et peuvent tre appliques au Paint Box.
c
 - Cette unit implmente galement les imprimantes qui ont une rsolution
 diffrente en horizontal et en vertical. Je n'ai malheureusement pas d'imprimante
 qui puisse tester cette nouvelle fonctionalit.

 - Une nouvelle procdure permet de configurer l'imprimante sans imprimer. Il est
 fortement conseill de passer par elle en lui transmettant la boite de dialogue de
 setup d'imprimante, car elle met  jour les polices d'imprimantes dont les
 caractristiques sont utilises par l'afficahge cran des textes.

 Libre  vous de modifier, amliorer, complter cette unit, si le coeur vous en
 dit. Et encore plus sympa confiez moi vos suggestions et j'essaierai d'en tenir
 compte et d'en faire profiter un maximum.
 }

INTERFACE

Uses StdCtrls, SysUtils,WinProcs, Classes, Graphics, Printers, Wintypes, Forms,
     Extctrls, Dialogs;

Type
   TJustificationH = (JtGauche,JtCentre,JtDroite,JtJustifie);
   TJustificationV = (JtHaut,JtMilieu,JtBas);
   TPeripherique = (Ecran,Imprimante);
   TEPParamEcranImprimante = record
      HauteurEcran,
      LargeurEcran,
      HauteurApercu,
      LargeurApercu,
      TopApercu,
      LeftApercu,
      PixelImpX,
      PixelImpY: Integer;
      HauteurPapierImp,
      LargeurPapierImp,
      HauteurImpression,
      LargeurImpression,
      MargeImpGauche,
      MargeImpHaute,
      MargeImpDroite,
      MargeImpBasse: Real;
   End;

{Procdures communes Ecran et Imprimante}

{Calcule les coefficients  appliquer pour afficher et imprimer  partir de valeurs
en cm}
Procedure EPCalculeCoeff(Sortie: TPeripherique;Loupe:Boolean);
{Dessine un rectangle avec un trait dfini par EPPen et une trame de fond dfinie par
EPBrush  partir de coordonnes en cm}
Procedure EPRectangle(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,XDroitCm,YBasCm: Real);
{Trace une ligne avec un trait dfini par EPPen  partir de coordonnes en cm}
Procedure EPLine(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,XDroitCm,YBasCm: Real);
{Ecrit une chaine de caractre avec la fonte dfinie par EPFont et sur le fond dfini par EPBrush
 partir d'un point dfini en cm}
Procedure EPTextout(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,XDroiteCm: real; Texte: string;
                        JustificationH: TJustificationH;Var XFinalCm,YFinalCm: Real);
{Dessine une image dans un rectangle de coordonnes dfinies en cm par point de dpart, largeur et hauteur}
Procedure EPDessinOut(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,LargeurCm,HauteurCm: Real;
                        var Dessin: TImage; JustificationH: TJustificationH;
                        JustificationV: TJustificationV;Var XFinalCm,YFinalCm: Real);
{Dfinit le style de trait, sa couleur et son paisseur en cm}
Procedure EPPen(Sortie: TPeripherique;var Image: TPaintBox; PColor: TColor; PWidth: real; PStyle: TPenStyle);
{Dfinit la couleur de fond et son style}
Procedure EPBrush(Sortie: TPeripherique;var Image: TPaintBox; BColor: TColor; BStyle: TBrushStyle);
{Dfinit la police du texte, sa couleur, sa taille et son style}
Procedure EPFont(Sortie: TPeripherique;var Image: TPaintBox; FName: TFontName; FColor: TColor; FSize: Integer;
                FStyle: TFontStyles);
{Renvoie la largeur du texte en cm}
Function EPTextWidth(Sortie: TPeripherique;var Image: TPaintBox; TexteAMesurer: String): Real;
{Renvoie la hauteur du texte en cm}
Function EPTextHeight(Sortie: TPeripherique;var Image: TPaintBox; TexteAMesurer: String): Real;
{Configure l'imprimante et met  jour les polices Indispensable pour l'affichage des textes }
Procedure EPConfigImprimante(Var SetupDialog: TPrinterSetupDialog;Loupe: Boolean);
Procedure EPImprime(Var PrintDialog: TPrintDialog; LaProcedure: TProcedure);



var
   CoeffEcran: real;         {Coefficient donnant le rapport entre la rsolution cran
                                et la taille en cm sur le PaintBox}
   CoeffX: real;             {Coefficient donnant le rapport entre la rsolution imprimante
                              horizontale et la taille en cm sur le papier}
   CoeffY: real;             {Coefficient donnant le rapport entre la rsolution imprimante
                              verticale et la taille en cm sur le papier}
   ParametresEcranImprimante: TEPParamEcranImprimante;

   {Conserv pour la compatibilit avec la version prcdente }

   LargeurEcran,             {Largeur de l'cran en pixels}
   Largeur: Integer;         {Largeur en pixel de la page imprimante}
   Hauteur: Integer;         {Hauteur en pixel de la page imprimante}
   LargeurPageA4,            {Largeur en pixel du PaintBox reprsentant une page A4}
   HauteurPageA4,            {Hauteur en pixel du PaintBox reprsentant une page A4}
   LeftPageA4,               {Position horizontale du PaintBox pour un A4}
   LargeurPageLetter,        {Largeur en pixel du PaintBox reprsentant une page format US}
   HauteurPageLetter,        {Hauteur en pixel du PaintBox reprsentant une page format US}
   LeftPageLetter: Integer;  {Position horizontale du PaintBox pour un format US}
   TalonGauche,              {Marge gauche o l'imprimante n'imprime pas}
   TalonHaut,                {Marge haute o l'imprimante n'imprime pas}
   TalonDroit,               {Marge droite o l'imprimante n'imprime pas}
   TalonBas,                 {Marge basse o l'imprimante n'imprime pas}
   Coeff: real;              {Maintenu pour la compatibilit avec la version prcdente
                             vaut CoeffX }

IMPLEMENTATION


var
   CoeffPolice,              {Coefficient qui permet de recalculer la taille de la police
                             en fonction de la taille du PaintBox}
   CoeffLoupe: real;         {Coefficient d'agrandissement du PaintBox pour que sa
                             largeur emplisse presque tout l'cran}


   { Taille du paint box pour un A4 en cran 1152 pixels sur 864 pixels
     820 de haut sur 580 de large
     ce qui donne les tailles suivantes de PaintBox proportionnels

     en A4 :  (29.7 x 21.0)

     1600 x 1200: 1156 x 817  Left  780   1      1.384   1.468   1.333  18   18.4   17
     1280 x 1024:  980 x 693  Left  584   0.8    1.306   1.477   1.250  15   14.72  14
     1152 x  864:  820 x 580  Left  569   0.72   1.404   1.489   1.333  12   13.248 12
     1024 x  768:  724 x 512  Left  509   0.64   1.414   1.500   1.333  10   11.776 10
     800  x  600:  556 x 393  Left  404   0.5    1.438   1.526   1.333   8    9.2    8
     640  x  480:  436 x 308  Left  329   0.4    1.467   1.558   1.333   6    7.36   6
                                      MOYENNE     1.402   1.503          1.5*hAUTEUR/100

     }

{Configure l'imprimante et met  jour les polices Indispensable pour l'affichage des textes }
Procedure EPImprime(Var PrintDialog: TPrintDialog; LaProcedure: TProcedure);
Begin
   If PrintDialog.Execute then          // Lance le dialogue de slection imprimante
      Begin
         EPCalculeCoeff(Imprimante,false); // Recalcule le coeff en cas de modif de rsolution
         Printer.BeginDoc;              // Initialise l'impression
         LaProcedure;       // Procdure qui encapsule le dessin de la page  l'imprimante
         Printer.EndDoc;                // Termine l'impression
      End;
End;


Procedure EPConfigImprimante(Var SetupDialog: TPrinterSetupDialog;Loupe: Boolean);
Begin
   If SetupDialog.Execute then
      Begin
         Printer.BeginDoc;  // Indispensable pour que le driver rende une hauteur
         Printer.Abort;     // et largeur de caractre correcte ncessaire  EPTextOut
         EPCalculeCoeff(Ecran,Loupe);
      End;
End;

{Calcule les coefficients  appliquer pour afficher et imprimer  partir de valeurs
en cm}
Procedure EPCalculeCoeff(Sortie: TPeripherique;Loupe:Boolean);
Var
   LargeurPage,
   HauteurPage: Integer;
Begin
   Case Sortie of  // Calcul du coefficient pour un cran
      Ecran:
         Begin
           If Loupe then            // Slection d'un coefficient d'agrandissement pour la loupe
              CoeffLoupe := 1.8965
           else
              CoeffLoupe := 1;
           EPCalculeCoeff(Imprimante,Loupe);
           LargeurEcran := Screen.Width;  // Dtection de la rsolution cran
           ParametresEcranImprimante.LargeurEcran := LargeurEcran;
           ParametresEcranImprimante.HauteurEcran := Screen.Height;
           If Printer.Orientation = PoPortrait then
              Begin
                 ParametresEcranImprimante.HauteurApercu := Trunc((Screen.Height-44)*CoeffLoupe);
                 ParametresEcranImprimante.LargeurApercu := Trunc((ParametresEcranImprimante.HauteurApercu *
                                     ParametresEcranImprimante.LargeurPapierImp) / ParametresEcranImprimante.HauteurPapierImp);
                 CoeffPolice := Round(((Screen.Height-44)*1.5/100)*CoeffLoupe*(29.7/ParametresEcranImprimante.HauteurPapierImp));
              End
           else
              Begin
                 ParametresEcranImprimante.LargeurApercu := Trunc((Screen.Height-44)*CoeffLoupe);
                 ParametresEcranImprimante.HauteurApercu := Trunc((ParametresEcranImprimante.LargeurApercu *
                                     ParametresEcranImprimante.HauteurPapierImp) / ParametresEcranImprimante.LargeurPapierImp);
                 CoeffPolice := Round(((Screen.Height-44)*1.5/100)*CoeffLoupe*(21/ParametresEcranImprimante.HauteurPapierImp));
              End;
           ParametresEcranImprimante.TopApercu := 10;
           If Loupe then
              ParametresEcranImprimante.LeftApercu := 10
           else
              ParametresEcranImprimante.LeftApercu := LargeurEcran - ParametresEcranImprimante.LargeurApercu - 20;
           CoeffEcran := ParametresEcranImprimante.HauteurApercu/ParametresEcranImprimante.HauteurPapierImp;

           If LargeurEcran = 1600 then    // Ecran de 1600 sur 1200
              Begin
//                 CoeffEcran := 1156*CoeffLoupe/29.7;    // Coefficient pour une page de 1156 pixel
                                                        // pour 29,7 CM
                 LargeurPageA4 := Trunc(817*CoeffLoupe);   // Calcul de la taille du PaintBox pour
                 HauteurPageA4 := Trunc(1156*CoeffLoupe);  // un A4 '21 x 29,7 cm)
                 If Loupe then
                    LeftPageA4 := 10                       // Positionnement du coin gauche suprieur
                                                           // du PaintBox pour un A4
                 else
                    LeftPageA4 := 763;
                 LargeurPageLetter := Trunc(893*CoeffLoupe);  // Calcul de la taille du PaintBox pour
                 HauteurPageLetter := Trunc(1156*CoeffLoupe); // un format Letter (21,6 x 27,9 cm)
                 If Loupe then
                    LeftPageLetter := 10
                 else                                      // Positionnement du coin gauche suprieur
                    LeftPageLetter := 687;                 // du PaintBox pour un format Letter
//                 CoeffPolice := 17*CoeffLoupe;             // Calcul du coefficient de grossissement de
              End;                                         // la police pour tre proportionnel au PaintBox
           If LargeurEcran = 1280 then       // Ecran de 1280 sur 1024
              Begin
//                 CoeffEcran := 980*CoeffLoupe/29.7;
                 LargeurPageA4 := Trunc(692*CoeffLoupe);
                 HauteurPageA4 := Trunc(980*CoeffLoupe);
                 If Loupe then
                    LeftPageA4 := 10
                 else
                    LeftPageA4 := 568;
                 LargeurPageLetter := Trunc(757*CoeffLoupe);
                 HauteurPageLetter := Trunc(980*CoeffLoupe);
                 If Loupe then
                    LeftPageLetter := 10
                 else
                    LeftPageLetter := 503;
//                 CoeffPolice := 14*CoeffLoupe;
              End;
           If LargeurEcran = 1152 then       // Ecran de 1152 x  864
              Begin
//                 CoeffEcran := 820*CoeffLoupe/29.7;
                 LargeurPageA4 := Trunc(579*CoeffLoupe);
                 HauteurPageA4 := Trunc(820*CoeffLoupe);
                 If Loupe then
                    LeftPageA4 := 10
                 else
                    LeftPageA4 := 553;
                 LargeurPageLetter := Trunc(633*CoeffLoupe);
                 HauteurPageLetter := Trunc(820*CoeffLoupe);
                 If Loupe then
                    LeftPageLetter := 10
                 else
                    LeftPageLetter := 499;
//                 CoeffPolice := 12*CoeffLoupe;
              End;
           If LargeurEcran = 1024 then       // Ecran de 1024 x 768
              Begin
//                 CoeffEcran := 724*CoeffLoupe/29.7;
                 LargeurPageA4 := Trunc(511*CoeffLoupe);
                 HauteurPageA4 := Trunc(724*CoeffLoupe);
                 If Loupe then
                    LeftPageA4 := 10
                 else
                    LeftPageA4 := 493;
                 LargeurPageLetter := Trunc(559*CoeffLoupe);
                 HauteurPageLetter := Trunc(724*CoeffLoupe);
                 If Loupe then
                    LeftPageLetter := 10
                 else
                    LeftPageLetter := 445;
//                 CoeffPolice := 11*CoeffLoupe;
              End;
           If LargeurEcran = 800 then      // Ecran de 800 x 600
              Begin
//                 CoeffEcran := 556*CoeffLoupe/29.7;
                 LargeurPageA4 := Trunc(393*CoeffLoupe);
                 HauteurPageA4 := Trunc(556*CoeffLoupe);
                 If Loupe then
                    LeftPageA4 := 10
                 else
                    LeftPageA4 := 387;
                 LargeurPageLetter := Trunc(429*CoeffLoupe);
                 HauteurPageLetter := Trunc(556*CoeffLoupe);
                 If Loupe then
                    LeftPageLetter := 10
                 else
                    LeftPageLetter := 351;
//                 CoeffPolice := 8*CoeffLoupe;
              End;
           If LargeurEcran = 640 then      // Ecran de 640 x 480
              Begin
//                 CoeffEcran := 436*CoeffLoupe/29.7;
                 LargeurPageA4 := Trunc(308*CoeffLoupe);
                 HauteurPageA4 := Trunc(436*CoeffLoupe);
                 If Loupe then
                    LeftPageA4 := 10
                 else
                    LeftPageA4 := 312;
                 LargeurPageLetter := Trunc(336*CoeffLoupe);
                 HauteurPageLetter := Trunc(436*CoeffLoupe);
                 If Loupe then
                    LeftPageLetter := 10
                 else
                    LeftPageLetter := 284;
//                 CoeffPolice := 6*CoeffLoupe;
              End;
         End;
      Imprimante:
         Begin
            HauteurPage := GetDeviceCaps(Printer.Handle,PHYSICALHEIGHT);
            LargeurPage := GetDeviceCaps(Printer.Handle,PHYSICALWIDTH);
            Largeur := Printer.PageWidth;   // Dtection de largeur de page de l'impression
            Hauteur := Printer.PageHeight;  // Dtection de hauteur de page de l'impression
            CoeffX := GetDeviceCaps(Printer.Handle,LOGPIXELSX)/2.54;
            Coeff := CoeffX;
            CoeffY := GetDeviceCaps(Printer.Handle,LOGPIXELSY)/2.54;
            TalonGauche := GetDeviceCaps(Printer.Handle,PHYSICALOFFSETX);   // Peut tre redfini en fonction de l'imprimante
            TalonHaut := GetDeviceCaps(Printer.Handle,PHYSICALOFFSETY);      // Peut tre redfini en fonction de l'imprimante
            TalonDroit := LargeurPage-Largeur-TalonGauche;
            TalonBas := HauteurPage-Hauteur-TalonHaut;
            TalonGauche := Talongauche/CoeffX;
            TalonHaut := TalonHaut/CoeffY;
            TalonDroit := TalonDroit/CoeffX;
            TalonBas := TalonBas/CoeffY;
            With ParametresEcranImprimante do
               begin
                  PixelImpX := GetDeviceCaps(Printer.Handle,LOGPIXELSX);
                  PixelImpY := GetDeviceCaps(Printer.Handle,LOGPIXELSY);
                  HauteurPapierImp := HauteurPage/CoeffY;
                  LargeurPapierImp := LargeurPage/CoeffX;
                  HauteurImpression := Hauteur/CoeffY;
                  LargeurImpression := Largeur/CoeffX;
                  MargeImpGauche := TalonGauche;
                  MargeImpHaute := TalonHaut;
                  MargeImpDroite := TalonDroit;
                  MargeImpBasse := TalonBas;
               End;
         End;
   End;
End;

{Dessine un rectangle avec un trait dfini par EPPen et une trame de fond dfinie par
EPBrush  partir de coordonnes en cm}
Procedure EPRectangle(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,XDroitCm,YBasCm: Real);
Begin
   Case Sortie of
      Ecran:
         Begin
            Image.Canvas.Rectangle(Trunc((XGaucheCm)*CoeffEcran),        // recalcule les coordonnes
                                           Trunc((YHautCm)*CoeffEcran),  // en pixels
                                           Trunc((XDroitCm)*CoeffEcran),
                                           Trunc((YBasCm)*CoeffEcran));
         End;
      Imprimante:
         Begin
            Printer.Canvas.Rectangle(Trunc((XGaucheCm - TalonGauche)*CoeffX), // recalcule les coordonnes
                                     Trunc((YHautCm - TalonHaut)*CoeffY),     // en pixels et retire les
                                     Trunc((XDroitCm - TalonGauche)*CoeffX),  // marges non imprimables
                                     Trunc((YBasCm - TalonHaut)*CoeffY));     // pour respecter le dpart
         End;                                                                // en bord de feuille
   End;
End;

{Trace une ligne avec un trait dfini par EPPen  partir de coordonnes en cm}
Procedure EPLine(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,XDroitCm,YBasCm: Real);
Begin
   Case Sortie of
      Ecran:
         Begin
            With Image.Canvas do
               begin                      // Se dplace en dpart de ligne
                  MoveTo(Trunc((XGaucheCm)*CoeffEcran),Trunc((YHautCm)*CoeffEcran));
                  LineTo(Trunc((XDroitCm)*CoeffEcran),Trunc((YBasCm)*CoeffEcran));
               End;                       // Tire la ligne
         End;
      Imprimante:
         Begin
            with Printer.Canvas do
               begin
                  MoveTo(Trunc((XGaucheCm - TalonGauche)*CoeffX),
                         Trunc((YHautCm - TalonHaut)*CoeffY));
                  LineTo(Trunc((XDroitCm - TalonGauche)*CoeffX),
                         Trunc((YBasCm - TalonHaut)*CoeffY));
               End;
         End;
   End;
End;

{Ecrit une chaine de caractre avec la fonte dfinie par EPFont et sur le fond dfini par EPBrush
 partir d'un point dfini en cm}
Procedure EPTextout(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,XDroiteCm: real; Texte: string;
                        JustificationH: TJustificationH;Var XFinalCm,YFinalCm: Real);
Var
   Long,        // sert pour le calcul de la longueur de chaine
   i: Integer;  // sert pour les boucles
   Delta: Real; // sert pour le calcul de justification
Begin
   Case Sortie of
      Ecran:
         Begin
            EPCalculeCoeff(Imprimante,False);  // On utilisera les tailles en cm renvoyes
                                               // par l'imprimante pour s'affranchir de
                                               // l'imprcision de la police cran
            Case JustificationH of      // pour un calage  gauche
               JtGauche: Begin
                               Long := Length(Texte); // on boucle en affichant chaque
                               For i := 1 to Long do  // caractre un  un
                                  Begin
                                     Image.Canvas.TextOut(Trunc((XGaucheCm)*CoeffEcran),
                                                   Trunc((YHautCm)*CoeffEcran),Texte[i]);
                                     XGaucheCm := XGaucheCm + EPTextWidth(Imprimante,Image,Texte[i]);
                                  End;                // et on se dcale de la largeur du caractre
                               XFinalCm := XGaucheCm; // puis on met  jour la posotion finale en x
                         End;
               JtCentre: Begin          // pour un calage au centre on retranche  x la moiti
                                        // de la longueur de chaine
                               XGaucheCm := XGaucheCm - EPTextWidth(Imprimante,Image,Texte)/2;
                               Long := Length(Texte);  // puis on boucle caractre par caractre
                               For i := 1 to Long do
                                  Begin
                                     Image.Canvas.TextOut(Trunc((XGaucheCm)*CoeffEcran),
                                                   Trunc((YHautCm)*CoeffEcran),Texte[i]);
                                     XGaucheCm := XGaucheCm + EPTextWidth(Imprimante,Image,Texte[i]);
                                  End;
                               XFinalCm := XGaucheCm;
                         End;
               JtDroite: Begin          // pour un calage  droite on retranche  x la longueur
                                        // de la chaine
                               XGaucheCm := XGaucheCm - EPTextWidth(Imprimante,Image,Texte);
                               Long := Length(Texte);
                               For i := 1 to Long do   // puis on boucle caractre par caractre
                                  Begin
                                     Image.Canvas.TextOut(Trunc((XGaucheCm)*CoeffEcran),
                                                   Trunc((YHautCm)*CoeffEcran),Texte[i]);
                                     XGaucheCm := XGaucheCm + EPTextWidth(Imprimante,Image,Texte[i]);
                                  End;
                               XFinalCm := XGaucheCm;
                         End;
               JtJustifie: Begin  // pour une justification, si le texte est moins long que l'espace
                                  // et la chaine non vide (vite la division par 0
                                  If (EPTextWidth(Sortie,Image,Texte) < (XDroiteCm - XGaucheCm))
                                    and (Length(Texte) <> 0) then
                                    Begin
                                       Long := Length(Texte); // on calcule l'espace  rajouter entre caractres
                                       Delta := (XDroiteCm - XGaucheCm - EPTextWidth(Imprimante,Image,Texte))/Long;
                                       For i := 1 to Long do  // puis on boucle
                                          Begin
                                             Image.Canvas.TextOut(Trunc((XGaucheCm)*CoeffEcran),
                                             Trunc((YHautCm)*CoeffEcran),Texte[i]);
                                             XGaucheCm := XGaucheCm + EPTextWidth(Imprimante,Image,Texte[i]) +
                                                          Delta;  // et on se dplace d'un caractre + le delta
                                             XFinalCm := XGaucheCm;
                                          End;
                                     End
                                  Else   // si la chaine est trop longue on se contente d'un calage  gauche
                                     Begin
                                        Long := Length(Texte);
                                        For i := 1 to Long do
                                           Begin
                                              Image.Canvas.TextOut(Trunc((XGaucheCm)*CoeffEcran),
                                                            Trunc((YHautCm)*CoeffEcran),Texte[i]);
                                              XGaucheCm := XGaucheCm + EPTextWidth(Imprimante,Image,Texte[i]);
                                           End;
                                        XFinalCm := XGaucheCm;
                                     End;
                           End;
               End;
            If Texte <> '' then  // puis on calcule le y final
               YFinalCm := YHautCm + EPTextHeight(Imprimante,Image,Texte)
            else
               YFinalCm := YHautCm + EPTextHeight(Imprimante,Image,'T'); // pour obtenir une hauteur
         End;                                                            // pour une chaine vide
      Imprimante:  // Mme calcul que pour l'cran, mais ici les boucles sont inutiles
         Begin     // sauf dans le cas de la justification
            Case JustificationH of

               JtGauche: Begin Printer.Canvas.TextOut(Trunc((XGaucheCm-TalonGauche)*CoeffX),
                               Trunc((YHautCm-TalonHaut)*CoeffY),Texte);
                               XFinalCm := XGaucheCm + EPTextWidth(Sortie,Image,Texte);
                         End;
               JtCentre: Begin XGaucheCm := XGaucheCm - EPTextWidth(Sortie,Image,Texte)/2;
                               Printer.Canvas.TextOut(Trunc((XGaucheCm-TalonGauche)*CoeffX),
                               Trunc((YHautCm-TalonHaut)*CoeffY),Texte);
                               XFinalCm := XGaucheCm + EPTextWidth(Sortie,Image,Texte);
                         End;
               JtDroite: Begin XGaucheCm := XGaucheCm - EPTextWidth(Sortie,Image,Texte);
                               Printer.Canvas.TextOut(Trunc((XGaucheCm-TalonGauche)*CoeffX),
                               Trunc((YHautCm-TalonHaut)*CoeffY),Texte);
                               XFinalCm := XGaucheCm + EPTextWidth(Sortie,Image,Texte);
                         End;
               JtJustifie: Begin If (EPTextWidth(Sortie,Image,Texte) < (XDroiteCm - XGaucheCm))
                                    and (Length(Texte) <> 0) then
                                    Begin
                                       Long := Length(Texte);
                                       Delta := (XDroiteCm - XGaucheCm - EPTextWidth(Sortie,Image,Texte))/Long;
                                       For i := 1 to Long do
                                          Begin
                                             Printer.Canvas.TextOut(Trunc((XGaucheCm-TalonGauche)*CoeffX),
                                             Trunc((YHautCm-TalonHaut)*CoeffY),Texte[i]);
                                             XGaucheCm := XGaucheCm + EPTextWidth(Sortie,Image,Texte[i]) +
                                                          Delta;
                                             XFinalCm := XGaucheCm;
                                          End;
                                    End
                                  Else
                                    Begin
                                       Printer.Canvas.TextOut(Trunc((XGaucheCm-TalonGauche)*CoeffX),
                                       Trunc((YHautCm-TalonHaut)*CoeffY),Texte);
                                       XFinalCm := XGaucheCm + EPTextWidth(Sortie,Image,Texte);
                                    End;
                           End;
               End;
            If Texte <> '' then
               YFinalCm := YHautCm + EPTextHeight(Sortie,Image,Texte)
            else
               YFinalCm := YHautCm + EPTextHeight(Sortie,Image,'T');
         End;
   End;
End;

{Dessine une image dans un rectangle de coordonnes dfinies en cm par point de dpart, largeur et hauteur}
Procedure EPDessinOut(Sortie: TPeripherique;var Image: TPaintBox; XGaucheCm,YHautCm,LargeurCm,HauteurCm: Real;
                        var Dessin: TImage; JustificationH: TJustificationH;
                        JustificationV: TJustificationV;Var XFinalCm,YFinalCm: Real);
Var
   LaZone: TRect;  // pour la dfinition du rectangle de l'image
Begin
   Case Sortie of
      Ecran:
         Begin // si largeur et hauteur sont nuls, on fait un dessin de largeur 20 cm
               // et proportionnel en hauteur  l'image
            If (LargeurCm = 0) and (HauteurCm = 0) then
               Begin
                  LargeurCm := 20;
                  HauteurCm := (Dessin.Picture.Height * LargeurCm)/Dessin.Picture.Width;
               End
            else  // si la largeur est nulle on la calcule proprtionnelle  l'image
               If LargeurCm = 0 then
                  Begin
                     LargeurCm := (Dessin.Picture.Width * HauteurCm)/Dessin.Picture.Height;
                  End
               Else  // si la hauteur est nulle on la calcule proprtionnelle  l'image
                  If HauteurCm = 0 then
                     Begin
                        HauteurCm := (Dessin.Picture.Height * LargeurCm)/Dessin.Picture.Width;
                     End;
            // dans les autres cas le dessin sera dform par le rectangle o il s'inscrit
            Case JustificationH of                             // horizontalement :
               JtCentre: XGaucheCm := XGaucheCm - LargeurCm/2; // le dessin sera centr sur le
                                                               // le point de dpart
               JtDroite: XGaucheCm := XGaucheCm - LargeurCm;   // le dessin sera cal  droite
                                                               // sur le point de dpart
            End;
            Case JustificationV of                             // verticalement :
               JtMilieu: YHautCm := YHautCm - HauteurCm/2;     // le dessin sera centr sur
                                                               // le point de dpart
               JtBas: YHautCm := YHautCm - HauteurCm;          // le dessin sera au dessus
                                                               // du point de dpart
            End;
            LaZone.Left := Trunc((XGaucheCm)*CoeffEcran);      // dfinit le rectangle
            LaZone.Top := Trunc((YHautCm)*CoeffEcran);
            LaZone.Right := Trunc((XGaucheCm+LargeurCm)*CoeffEcran);
            LaZone.Bottom := Trunc((YHautCm+HauteurCm)*CoeffEcran);
            With Image.Canvas do
               Begin
                  StretchDraw(LaZone,Dessin.Picture.Graphic);  // dessine l'image dans le rectangle
               End;
            XFinalCm := XGaucheCm+LargeurCm;                   // recalcule x et y finaux au point bas
            YFinalCm := YHautCm+HauteurCm;                     // et droite de l'image
         End;
      Imprimante:    // mme procdure que pour l'cran en retranchant les talons de non impression
         Begin
            If (LargeurCm = 0) and (HauteurCm = 0) then
               Begin
                  LargeurCm := 20;
                  HauteurCm := (Dessin.Picture.Height * LargeurCm)/Dessin.Picture.Width;
               End
            else
               If LargeurCm = 0 then
                  Begin
                     LargeurCm := (Dessin.Picture.Width * HauteurCm)/Dessin.Picture.Height;
                  End
               Else
                  If HauteurCm = 0 then
                     Begin
                        HauteurCm := (Dessin.Picture.Height * LargeurCm)/Dessin.Picture.Width;
                     End;
            Case JustificationH of
               JtCentre: XGaucheCm := XGaucheCm - LargeurCm/2;
               JtDroite: XGaucheCm := XGaucheCm - LargeurCm;
            End;
            Case JustificationV of
               JtMilieu: YHautCm := YHautCm - HauteurCm/2;
               JtBas: YHautCm := YHautCm - HauteurCm;
            End;
            LaZone.Left := Trunc((XGaucheCm-TalonGauche)*CoeffX);
            LaZone.Top := Trunc((YHautCm-TalonHaut)*CoeffY);
            LaZone.Right := Trunc((XGaucheCm+LargeurCm-TalonGauche)*CoeffX);
            LaZone.Bottom := Trunc((YHautCm+HauteurCm-TalonHaut)*CoeffY);
            Printer.Canvas.StretchDraw(LaZone,Dessin.Picture.Graphic);
            Printer.Canvas.StretchDraw(LaZone,Dessin.Picture.Graphic);
            XFinalCm := XGaucheCm+LargeurCm;
            YFinalCm := YHautCm+HauteurCm;
         End;
   End;
End;

{Dfinit le style de trait,sa couleur et son paisseur en cm}
Procedure EPPen(Sortie: TPeripherique;var Image: TPaintBox; PColor: TColor; PWidth: real; PStyle: TPenStyle);
Begin
   Case Sortie of
      Ecran:
         Begin
            with Image.Canvas.Pen do
               Begin
                  Color := PColor;
                  Width := Trunc(PWidth * CoeffEcran); // Pour l'cran on applique
                                                       // le coefficient cran  la largeur de trait
                  Style := PStyle;
               End;
         End;
      Imprimante:
         Begin
            with Printer.Canvas.Pen do
               Begin
                  Color := PColor;
                  Width := Trunc(PWidth * CoeffX); // Pour l'imprimante on applique
                                                  // le coefficient imprimante  la largeur de trait
                  Style := PStyle;
               End;
         End;
   End;
End;

{Dfinit la couleur de fond et son style}
Procedure EPBrush(Sortie: TPeripherique;var Image: TPaintBox; BColor: TColor; BStyle: TBrushStyle);
Begin
   Case Sortie of
      Ecran:
         Begin
            with Image.Canvas.Brush do
               Begin
                  Color := BColor;
                  Style := BStyle;
               End;
         End;
      Imprimante:
         Begin
            with Printer.Canvas.Brush do
               Begin
                  Color := BColor;
                  Style := BStyle;
               End;
         End;
   End;
End;

{Dfinit la police du texte, sa couleur, sa taille et son style}
Procedure EPFont(Sortie: TPeripherique;var Image: TPaintBox; FName: TFontName; FColor: TColor; FSize: Integer;
                FStyle: TFontStyles);
Begin
   Case Sortie of
      Ecran:  // pour l'cran on recalcule la taille de la fonte la plus approchante
         Begin
            with Image.Canvas.Font do
               Begin
                  Name := FName;
                  Color := FColor;
                  Style := FStyle;
                  Height := -Trunc((CoeffPolice*FSize)/12);
               End;
            with Printer.Canvas.Font do  // puis pour des calculs plus prcis du texte
                                         // on dfinit la mme police en impression
               Begin
                  Name := FName;
                  Color := FColor;
                  Size := FSize;
                  Style := FStyle;
               End;
         End;
      Imprimante:
         Begin
            with Printer.Canvas.Font do
               Begin
                  Name := FName;
                  Color := FColor;
                  Size := FSize;
                  Style := FStyle;
               End;
         End;
   End;
End;

{Renvoie la largeur du texte en cm}
Function EPTextWidth(Sortie: TPeripherique;var Image: TPaintBox; TexteAMesurer: String): Real;
Begin
   Case Sortie of
      Ecran:       // renvoie la longueur du texte en cm de la reprsentation cran
         Begin
            With Image.Canvas do
               Begin
                  Result := TextWidth(TexteAMesurer)/CoeffEcran;
               End;
         End;
      Imprimante:  // renvoie la longueur du texte en cm de la page imprime
         Begin
            With Printer.Canvas do
               Begin
                  Result := TextWidth(TexteAMesurer)/CoeffX;
               End;
         End;
   End;
End;

{Renvoie la hauteur du texte en cm}
Function EPTextHeight(Sortie: TPeripherique;var Image: TPaintBox; TexteAMesurer: String): Real;
Begin
   Case Sortie of
      Ecran:   // renvoie la hauteur de ligne en cm de la reprsentation cran
         Begin
            With Image.Canvas do
               Begin
                  Result := TextHeight(TexteAMesurer)/CoeffEcran;
               End;
         End;
      Imprimante:  // renvoie la hauteur de ligne en cm de la page imprime
         Begin
            With Printer.Canvas do
               Begin
                  Result := TextHeight(TexteAMesurer)/CoeffY;
               End;
         End;
   End;
End;


Initialization
   EpCalculeCoeff(Ecran,false);

End.
